autoPtr<surfaceScalarField> heatFlux;

autoPtr<basicThermo> thermo
(
    basicThermo::NewIfPresent(mesh,word::null,false)
);

autoPtr<volVectorField> UPtr;
autoPtr<surfaceScalarField> phiPtr;

if( (!thermo.valid()) || isA<fluidThermo>(thermo()) )
{
    UPtr.reset
    (
        new volVectorField
        (
            IOobject
            (
                "U",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::AUTO_WRITE
            ),
            mesh
        )
    );
}

// Construct turbulence model (if fluid)
autoPtr<volScalarField> rhoPtr;
autoPtr<compressible::turbulenceModel> compressibleTurbulence;

if (thermo.valid() && isA<fluidThermo>(thermo()))
{
    // Register copy of thermo density
    rhoPtr.reset
    (
        new volScalarField
        (
            IOobject
            (
                "rho",
                runTime.timeName(),
                mesh
            ),
            thermo->rho()
        )
    );

    const volScalarField rho = rhoPtr();
    const volVectorField U = UPtr();

    #include "compressibleCreatePhi.H"
    // Copy phi to autoPtr. Rename to make sure copy is now registered as 'phi'.
    phi.rename("phiFluid");
    phiPtr.reset(new surfaceScalarField("phi", phi));

    compressibleTurbulence = compressible::turbulenceModel::New
    (
        rhoPtr(),
        UPtr(),
        phiPtr(),
        refCast<const fluidThermo>(thermo())
    );
}

//If thermo package not found try Boussinesq
autoPtr<volScalarField> T;
autoPtr<volScalarField> alphat;
autoPtr<volScalarField> rhok;

autoPtr<incompressible::turbulenceModel> incompressibleTurbulence;
autoPtr<singlePhaseTransportModel> laminarTransport;

dimensionedScalar Pr("Pr", dimless, 1);
dimensionedScalar rhoRef("rhoRef", dimDensity, 0);
dimensionedScalar CpRef("CpRef", dimSpecificHeatCapacity, 0);

if(!thermo.valid())
{
    T.reset
    (
        new volScalarField
        (
            IOobject
            (
                "T",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::NO_WRITE
            ),
            mesh
        )
    );

    alphat.reset
    (
        new volScalarField
        (
            IOobject
            (
                "alphat",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::NO_WRITE
            ),
            mesh
        )
    );

    rhok.reset
    (
        new volScalarField
        (
            IOobject
            (
                "rhok",
                runTime.timeName(),
                mesh,
                IOobject::READ_IF_PRESENT,
                IOobject::NO_WRITE
            ),
            mesh,
            dimensionedScalar("rhok",dimless,1.0)
        )
    );

    const volVectorField U = UPtr();

    #include "createPhi.H"

    phi.rename("phiFluid");
    phiPtr.reset(new surfaceScalarField("phi", phi));

    laminarTransport.reset
    ( 
        new singlePhaseTransportModel(UPtr(), phiPtr())
    );

    incompressibleTurbulence = incompressible::turbulenceModel::New
    (
        UPtr(), 
        phiPtr(), 
        laminarTransport()
    );

    laminarTransport().lookup("Pr") >> Pr;
    laminarTransport().lookup("rhoRef") >> rhoRef;
    laminarTransport().lookup("CpRef") >> CpRef;
}

//Read radiative heat flux if available
volScalarField Qr
(
    IOobject
    (
        "Qr",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::NO_WRITE
    ),
    mesh,
    dimensionedScalar("Qr", dimMass/pow3(dimTime), 0.0)
);

